@@ -1,5 +1,5 @@
 package Terse;
-our $VERSION = '0.112';
+our $VERSION = '0.113';
 use 5.006;
 use strict;
 use warnings;
@@ -79,8 +79,13 @@ sub run {
 					}
 					$response = $websocket->recieve();
 					if ($response) {
-						$ping_rate = $reset_rate;
-						$args{recieve}->($websocket, $response, $responder);
+						if ($response =~ m/^invalid_(length|version|host|required_key)$/) {
+							$args{error}->($websocket, $response, $responder);
+							last;
+						} else {
+							$ping_rate = $reset_rate;
+							$args{recieve}->($websocket, $response, $responder);
+						}
 					}
 				}
 			};
@@ -368,7 +373,7 @@ Terse - Lightweight Web Framework
 
 =head1 VERSION
 
-Version 0.112
+Version 0.113
 
 =cut
 
