@@ -52,7 +52,7 @@ sub headers {
 	push @headers, Upgrade => 'WebSocket';
 	push @headers, Connection => 'Upgrade';
 	if ($version eq 'draft-hixie-75' || $version eq 'draft-ietf-hybi-00') {
-        	die(qq/host is required/) unless defined $self->options->host;
+        	return 'invalid_host' unless defined $self->options->host;
 		my $location = 'ws';
     		$location .= 's' if $self->options->secure;
     		$location .= '://';
@@ -75,7 +75,7 @@ sub headers {
         	}
     	}
 	elsif ($version eq 'draft-ietf-hybi-10' || $version eq 'draft-ietf-hybi-17') {
-        	die(qq/key is required/) unless defined $self->options->key;
+        	return 'invalid_required_key' unless defined $self->options->key;
         	my $key = $self->options->key;
         	$key .= $self->options->secret;
         	$key = Digest::SHA::sha1($key);
@@ -86,7 +86,7 @@ sub headers {
           		if defined $self->options->subprotocol;
     	}
     	else {
-        	die('Version ' . $version . ' is not supported');
+		return 'invalid_version';
     	}
 	return @headers;
 }
@@ -94,13 +94,13 @@ sub headers {
 sub send {
 	my ($self, $message) = @_;
 	my $pg = $self->psgix;	
-	my $mask =  1;
+	my $mask = 1; $self->mask;
 	my (@DECODED) = map { ord($_) } split //, $message;
 	my @ENCODED;
 	my @MASK;
 	my $length = scalar @DECODED + 128;
 	if ($length > 256 || $mask) {
-		@MASK =  map { int(rand(256)) } 0 .. 3;
+		@MASK = (1, 1, 1, 1); # map { int(rand(256)) } 0 .. 3;
 		my $i;
 		for (@DECODED) {
 			push @ENCODED, $_ ^ $MASK[$i++ % 4];
@@ -119,7 +119,6 @@ sub send {
 		@ENCODED = @DECODED;
 		unshift @ENCODED, $length;
 	}
-	my $dead = $mask + $length;
 	syswrite $pg, join("", map {chr($_)} (128 + ($mask ? 1 : 0),  @ENCODED));
 	return;
 }
@@ -131,7 +130,6 @@ sub recieve {
 	my $len = sysread($pg, $content, 8192);
 	return unless $len;
 	$len = sysread($pg, $content, 8192, length($content)) while $len >= 8192;
-	my @ENC = map { unpack "C", $_ } split //, $content;
 	my $hdr = substr($content, 0, 1);
 	my @bits = split //, unpack("B*", $hdr);
         $self->fin($bits[0]);
@@ -141,14 +139,14 @@ sub recieve {
 	my $length;
 	if (scalar @ENCODED > 126) {
 		my @length = splice @ENCODED, 0, 3;
-		$length = (($length[0] + 2) * $length[1]) + $length[2];
+		$length = ((($length[0] + 2) * $length[1]) + $length[2]);
 	} else {
 		$length = shift @ENCODED;
-		$length -= 127;
+		$length -= 128;
 	}
 	my @MASK = splice @ENCODED, 0, 4;
-	my @DECODED;
-	my $i;
+	return 'invalid_length' if (scalar @ENCODED != $length);
+	my (@DECODED, $i);
 	for (@ENCODED) {
 		push @DECODED, $_ ^ $MASK[$i++ % 4];
 	}
@@ -166,7 +164,7 @@ Terse::WebSocket - Lightweight WebSockets
 
 =head1 VERSION
 
-Version 0.112
+Version 0.113
 
 =cut
 
